@@ -5,15 +5,17 @@ from ...activations import ACT_RELU, get_activation_block
 __all__ = ["HRNetW18Encoder", "HRNetW32Encoder", "HRNetW48Encoder", "TimmHRNetW18SmallV2Encoder"]
 
 
-class HRNetW18Encoder(EncoderModule):
-    def __init__(self, pretrained=True, layers=None):
+class HRNetW18Encoder(GenericTimmEncoder):
+    def __init__(self, pretrained=True, use_incre_features: bool = True, layers=None):
         from timm.models import hrnet
 
-        if layers is None:
-            layers = [1, 2, 3, 4]
-        encoder = hrnet.hrnet_w18(pretrained=pretrained, features_only=True, out_indices=(0, 1, 2, 3, 4))
-        super().__init__([64, 128, 256, 512, 1024], [2, 4, 8, 16, 32], layers)
-        self.encoder = encoder
+        encoder = hrnet.hrnet_w18(
+            pretrained=pretrained,
+            feature_location="incre" if use_incre_features else "",
+            features_only=True,
+            out_indices=(0, 1, 2, 3, 4),
+        )
+        super().__init__(encoder, layers)
 
     def forward(self, x):
         y = self.encoder.forward(x)
@@ -24,16 +26,17 @@ class HRNetW18Encoder(EncoderModule):
         return self
 
 
-class HRNetW32Encoder(EncoderModule):
-    def __init__(self, pretrained=True, layers=None):
+class HRNetW32Encoder(GenericTimmEncoder):
+    def __init__(self, pretrained=True, use_incre_features: bool = True, layers=None):
         from timm.models import hrnet
 
-        if layers is None:
-            layers = [1, 2, 3, 4]
-
-        encoder = hrnet.hrnet_w32(pretrained=pretrained, features_only=True, out_indices=(0, 1, 2, 3, 4))
-        super().__init__([64, 128, 256, 512, 1024], [2, 4, 8, 16, 32], layers)
-        self.encoder = encoder
+        encoder = hrnet.hrnet_w32(
+            pretrained=pretrained,
+            feature_location="incre" if use_incre_features else "",
+            features_only=True,
+            out_indices=(0, 1, 2, 3, 4),
+        )
+        super().__init__(encoder, layers)
 
     def forward(self, x):
         y = self.encoder.forward(x)
@@ -44,16 +47,17 @@ class HRNetW32Encoder(EncoderModule):
         return self
 
 
-class HRNetW48Encoder(EncoderModule):
-    def __init__(self, pretrained=True, layers=None):
+class HRNetW48Encoder(GenericTimmEncoder):
+    def __init__(self, pretrained=True, use_incre_features: bool = True, layers=None):
         from timm.models import hrnet
 
-        if layers is None:
-            layers = [1, 2, 3, 4]
-
-        encoder = hrnet.hrnet_w48(pretrained=pretrained, features_only=True, out_indices=(0, 1, 2, 3, 4))
-        super().__init__([64, 128, 256, 512, 1024], [2, 4, 8, 16, 32], layers)
-        self.encoder = encoder
+        encoder = hrnet.hrnet_w48(
+            pretrained=pretrained,
+            feature_location="incre" if use_incre_features else "",
+            features_only=True,
+            out_indices=(0, 1, 2, 3, 4),
+        )
+        super().__init__(encoder, layers)
 
     def forward(self, x):
         y = self.encoder.forward(x)
@@ -65,10 +69,15 @@ class HRNetW48Encoder(EncoderModule):
 
 
 class TimmHRNetW18SmallV2Encoder(GenericTimmEncoder):
-    def __init__(self, pretrained=True, layers=None, activation=ACT_RELU):
+    def __init__(self, pretrained=True, use_incre_features: bool = True, layers=None, activation=ACT_RELU):
         from timm.models import hrnet
 
-        encoder = hrnet.hrnet_w18_small_v2(pretrained=pretrained, features_only=True, out_indices=(0, 1, 2, 3, 4))
+        encoder = hrnet.hrnet_w18_small_v2(
+            pretrained=pretrained,
+            feature_location="incre" if use_incre_features else "",
+            features_only=True,
+            out_indices=(0, 1, 2, 3, 4),
+        )
         super().__init__(encoder, layers)
 
     def change_input_channels(self, input_channels: int, mode="auto", **kwargs):
